@@ -1,5 +1,5 @@
 # This file is part of Wike (com.github.hugolabe.Wike)
-# SPDX-FileCopyrightText: 2021-23 Hugo Olabera <hugolabe@gmail.com>
+# SPDX-FileCopyrightText: 2021-24 Hugo Olabera <hugolabe@gmail.com>
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 
@@ -10,16 +10,18 @@ from wike.data import languages, bookmarks
 
 # Box bookmarks in sidebar
 
-@Gtk.Template(resource_path='/com/github/hugolabe/Wike/ui/bookmarks.ui')
-class BookmarksBox(Gtk.Box):
+@Gtk.Template(resource_path='/com/github/hugolabe/Wike/gtk/bookmarks.ui')
+class BookmarksPanel(Adw.Bin):
 
-  __gtype_name__ = 'BookmarksBox'
+  __gtype_name__ = 'BookmarksPanel'
 
   booklists_dropdown = Gtk.Template.Child()
+  selection_button = Gtk.Template.Child()
   menu_button = Gtk.Template.Child()
+  select_all_button = Gtk.Template.Child()
+  select_none_button = Gtk.Template.Child()
+  remove_button = Gtk.Template.Child()
   bookmarks_list = Gtk.Template.Child()
-  bookmarks_scroller = Gtk.Template.Child()
-  bookmarks_separator = Gtk.Template.Child()
 
   # Initialize widgets and connect signals
 
@@ -54,8 +56,10 @@ class BookmarksBox(Gtk.Box):
 
     self.booklists_dropdown.connect('notify::selected', self._booklists_dropdown_selected_cb)
     self.booklists_model.connect('items-changed', self._booklists_model_changed_cb)
+    self.select_all_button.connect('clicked', self._select_buttons_cb, True)
+    self.select_none_button.connect('clicked', self._select_buttons_cb, False)
+    self.remove_button.connect('clicked', self._remove_button_cb)
     self.bookmarks_list.connect('row-activated', self._list_activated_cb)
-    self.bookmarks_scroller.get_vadjustment().connect('value-changed', self._bookmarks_scrolled_cb)
 
   # Set actions for bookmarks menu
 
@@ -78,7 +82,6 @@ class BookmarksBox(Gtk.Box):
   def _booklists_factory_setup(self, factory, list_item):
     label = Gtk.Label()
     label.set_xalign(0)
-    label.set_max_width_chars(15)
     label.set_ellipsize(Pango.EllipsizeMode.END)
     list_item.set_child(label)
 
@@ -120,6 +123,8 @@ class BookmarksBox(Gtk.Box):
   # Populate bookmarks list
 
   def _populate(self, list_name):
+    self._clear_list()
+
     if list_name:
       bookmarks_items = bookmarks.lists[list_name]
     else:
@@ -130,11 +135,13 @@ class BookmarksBox(Gtk.Box):
       lang = bookmarks_items[bookmark][1]
       row = BookmarksRow(bookmark, title, lang)
       self.bookmarks_list.append(row)
-      row.remove_button.connect('clicked', self._row_remove_button_cb, row)
+      self.selection_button.bind_property('active', row.select_check, 'visible', Gio.SettingsBindFlags.DEFAULT)
 
   # Clear bookmarks list
 
-  def _clear(self):
+  def _clear_list(self):
+    self.selection_button.set_active(False)
+
     while True:
       row = self.bookmarks_list.get_row_at_index(0)
       if row:
@@ -165,16 +172,17 @@ class BookmarksBox(Gtk.Box):
 
   def add_bookmark(self, uri, title, lang):
     if bookmarks.add(uri, title, lang, self._booklist):
+      self.selection_button.set_active(False)
       row = BookmarksRow(uri, title, lang)
       self.bookmarks_list.append(row)
-      row.remove_button.connect('clicked', self._row_remove_button_cb, row)
+      self.selection_button.bind_property('active', row.select_check, 'visible', Gio.SettingsBindFlags.DEFAULT)
       self.refresh_buttons()
       return True
     else:
       return False
 
   # Remove bookmark from current list
-  
+
   def remove_bookmark(self, uri):
     if bookmarks.remove(uri, self._booklist):
       i = 0
@@ -194,15 +202,14 @@ class BookmarksBox(Gtk.Box):
 
   def _create_booklist_cb(self, action, parameter):
     builder = Gtk.Builder()
-    builder.add_from_resource('/com/github/hugolabe/Wike/ui/dialogs.ui')
+    builder.add_from_resource('/com/github/hugolabe/Wike/gtk/dialogs.ui')
     new_booklist_dialog = builder.get_object('new_booklist_dialog')
     name_entry = builder.get_object('name_entry')
-    new_booklist_dialog.set_transient_for(self._window)
 
     new_booklist_dialog.connect('response', self._create_booklist_response_cb, name_entry)
     name_entry.connect('changed', self._create_entry_changed_cb, new_booklist_dialog)
 
-    new_booklist_dialog.show()
+    new_booklist_dialog.present(self._window)
 
   # On response create bookmarks list
 
@@ -233,17 +240,16 @@ class BookmarksBox(Gtk.Box):
 
   def _rename_booklist_cb(self, action, parameter):
     builder = Gtk.Builder()
-    builder.add_from_resource('/com/github/hugolabe/Wike/ui/dialogs.ui')
+    builder.add_from_resource('/com/github/hugolabe/Wike/gtk/dialogs.ui')
     rename_booklist_dialog = builder.get_object('rename_booklist_dialog')
     rename_entry = builder.get_object('rename_entry')
-    rename_booklist_dialog.set_transient_for(self._window)
 
     rename_entry.set_text(self._booklist)
 
     rename_booklist_dialog.connect('response', self._rename_booklist_response_cb, rename_entry)
     rename_entry.connect('changed', self._rename_entry_changed_cb, rename_booklist_dialog)
 
-    rename_booklist_dialog.show()
+    rename_booklist_dialog.present(self._window)
 
   # On response rename bookmarks list
 
@@ -279,20 +285,19 @@ class BookmarksBox(Gtk.Box):
 
   def _clear_booklist_cb(self, action, parameter):
     builder = Gtk.Builder()
-    builder.add_from_resource('/com/github/hugolabe/Wike/ui/dialogs.ui')
+    builder.add_from_resource('/com/github/hugolabe/Wike/gtk/dialogs.ui')
     clear_booklist_dialog = builder.get_object('clear_booklist_dialog')
     delete_booklist_check = builder.get_object('delete_booklist_check')
-    clear_booklist_dialog.set_transient_for(self._window)
 
     if self.booklists_dropdown.get_selected() == 0:
       delete_booklist_check.set_sensitive(False)
 
     clear_booklist_dialog.connect('response', self._clear_booklist_response_cb, delete_booklist_check)
 
-    clear_booklist_dialog.show()
+    clear_booklist_dialog.present(self._window)
 
   # On response clear all bookmarks of current list
-  
+
   def _clear_booklist_response_cb(self, dialog, response, delete_booklist_check):
     if response == 'clear':
       if delete_booklist_check.get_active():
@@ -300,7 +305,7 @@ class BookmarksBox(Gtk.Box):
           self.booklists_model.remove(self.booklists_dropdown.get_selected())
       else:
         bookmarks.clear_list(self._booklist)
-        self._clear()
+        self._clear_list()
         self.refresh_buttons()
 
   # Dropdown list selected changed
@@ -313,7 +318,6 @@ class BookmarksBox(Gtk.Box):
     else:
       self._booklist = list_name
 
-    self._clear()
     self._populate(self._booklist)
     self.refresh_buttons()
 
@@ -326,29 +330,56 @@ class BookmarksBox(Gtk.Box):
       if removed == 1:
         self.booklists_dropdown.set_selected(0)
 
-  # On list activated load article in view
-
-  def _list_activated_cb(self, bookmarks_list, row):
-    if self._window.flap.get_fold_policy() == Adw.FlapFoldPolicy.ALWAYS:
-      self._window.flap.set_reveal_flap(False)
-
-    self._window.page.wikiview.load_wiki(row.uri)
-
-  # On row button remove bookmark and refresh buttons state
+  # On select all or select none buttons do selection
   
-  def _row_remove_button_cb(self, remove_button, row):
+  def _select_buttons_cb(self, button, select_mode):
+    i = 0
+    while True:
+      row = self.bookmarks_list.get_row_at_index(i)
+      if row:
+        if row.get_activatable():
+          row.select_check.set_active(select_mode)
+        i += 1
+      else:
+        break
+
+  # On button clicked remove selected items
+
+  def _remove_button_cb(self, button):
+    selected_rows = []
+    i = 0
+    while True:
+      row = self.bookmarks_list.get_row_at_index(i)
+      if row:
+        if row.select_check.get_active():
+          selected_rows.append(row)
+        i += 1
+      else:
+        break
+        
+    if len(selected_rows) > 0:
+      for row in selected_rows:
+        self._remove_row(row)
+
+  # Remove row from bookmarks and list
+
+  def _remove_row(self, row):
     if bookmarks.remove(row.uri, self._booklist):
       self.bookmarks_list.remove(row)
 
     self.refresh_buttons()
 
-  # Show/hide separator on scroll
+  # On list activated load article in view
 
-  def _bookmarks_scrolled_cb(self, adjustment):
-    if adjustment.get_value() > 0:
-      self.bookmarks_separator.set_visible(True)
-    else:
-      self.bookmarks_separator.set_visible(False)
+  def _list_activated_cb(self, bookmarks_list, row):
+    if self.selection_button.get_active():
+      row.select_check.set_active(not row.select_check.get_active())
+      return
+
+    if self._window.panel_split.get_collapsed():
+      self._window.panel_split.set_show_sidebar(False)
+
+    self._window.page.wikiview.load_wiki(row.uri)
 
 
 # This object represent a bookmarks list in dropdown
@@ -372,14 +403,14 @@ class Booklist(GObject.Object):
 
 # Row on bookmarks list
 
-@Gtk.Template(resource_path='/com/github/hugolabe/Wike/ui/bookmarks-row.ui')
+@Gtk.Template(resource_path='/com/github/hugolabe/Wike/gtk/bookmarks-row.ui')
 class BookmarksRow(Gtk.ListBoxRow):
 
   __gtype_name__ = 'BookmarksRow'
   
   title_label = Gtk.Template.Child()
   lang_label = Gtk.Template.Child()
-  remove_button = Gtk.Template.Child()
+  select_check = Gtk.Template.Child()
 
   # Set values and connect signals
 
@@ -396,7 +427,7 @@ class BookmarksRow(Gtk.ListBoxRow):
       lang_name = lang
 
     self.title_label.set_label(title)
-    self.lang_label.set_markup('<small>' + lang_name + '</small>')
+    self.lang_label.set_label(lang_name)
 
 
 # Popover menu for bookmarks options
@@ -405,22 +436,22 @@ class BookmarksMenuPopover(Gtk.PopoverMenu):
 
   # Set menu model and connect signals
 
-  def __init__(self, bookmarks_box):
+  def __init__(self, bookmarks_panel):
     super().__init__()
 
     builder_menu = Gtk.Builder()
-    builder_menu.add_from_resource('/com/github/hugolabe/Wike/ui/bookmarks-menu.ui')
+    builder_menu.add_from_resource('/com/github/hugolabe/Wike/gtk/bookmarks-menu.ui')
     menu = builder_menu.get_object('bookmarks_menu')
     self.set_menu_model(menu)
 
-    self.connect('show', self._popover_show_cb, bookmarks_box)
+    self.connect('show', self._popover_show_cb, bookmarks_panel)
 
   # Enable or disable menu items on popover show
 
-  def _popover_show_cb(self, popover, bookmarks_box):
-    rename_list_action = bookmarks_box.actions_group.lookup_action('rename-list')
+  def _popover_show_cb(self, popover, bookmarks_panel):
+    rename_list_action = bookmarks_panel.actions_group.lookup_action('rename-list')
 
-    if bookmarks_box.booklists_dropdown.get_selected() == 0:
+    if bookmarks_panel.booklists_dropdown.get_selected() == 0:
       rename_list_action.set_enabled(False)
     else:
       rename_list_action.set_enabled(True)
